﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;

namespace JsonValueTests
{
    [TestClass]
    public class JValueTests
    {
        [TestMethod]
        public void JObjectOutputTest()
        {
            // strong type instance 
            var jsonObject = new JObject();

            // you can explicitly add values here
            jsonObject.Add("Entered", DateTime.Now);

            // dynamic expando instance you can add properties to
            dynamic album = jsonObject;

            album.AlbumName = "Dirty Deeds Done Dirt Cheap";
            album.Artist = "AC/DC";
            album.YearReleased = 1976;

            album.Songs = new JArray() as dynamic;

            dynamic song = new JObject();
            song.SongName = "Dirty Deeds Done Dirt Cheap";
            song.SongLength = "4:11";
            album.Songs.Add(song);

            song = new JObject();
            song.SongName = "Love at First Feel";
            song.SongLength = "3:10";
            album.Songs.Add(song);

            Console.WriteLine(album.ToString());
        }

        [TestMethod]
        public void JsonValueParsingTest()
        {
            var jsonString = @"{""Name"":""Rick"",""Company"":""West Wind"",""Entered"":""2012-03-16T00:03:33.245-10:00""}";

            dynamic json = JValue.Parse(jsonString);
            //dynamic json = jsonVal;

            // values require casting
            string name = json.Name;
            string company = json.Company;
            DateTime entered = json.Entered;

            Assert.AreEqual(name, "Rick");
            Assert.AreEqual(company, "West Wind");
        }

        [TestMethod]
        public void JsonArrayParsingTest()
        {
            JArray jsonVal = JArray.Parse(jsonString) as JArray;
            dynamic albums = jsonVal;

            foreach (dynamic album in albums)
            {
                Console.WriteLine(album.AlbumName + " (" + album.YearReleased.ToString() + ")");
                foreach (dynamic song in album.Songs)
                {
                    Console.WriteLine("\t" + song.SongName);
                }
            }

            Console.WriteLine(albums[0].AlbumName);
            Console.WriteLine(albums[0].Songs[1].SongName);
        }

        [TestMethod]
        public void JsonParseToStrongTypeTest()
        {
            JArray albums = JArray.Parse(jsonString) as JArray;

            // pick out one album
            JObject jalbum = albums[0] as JObject;

            // Copy to a static Album instance
            Album album = jalbum.ToObject<Album>();

            Assert.IsNotNull(album);
            Assert.AreEqual(album.AlbumName, jalbum.Value<string>("AlbumName"));
            Assert.IsTrue(album.Songs.Count > 0);
        }



        [TestMethod]
        public void StronglyTypedSerializationTest()
        {

            // Demonstrate deserialization from a raw string
            var album = new Album()
            {
                AlbumName = "Dirty Deeds Done Dirt Cheap",
                Artist = "AC/DC",
                Entered = DateTime.Now,
                YearReleased = 1976,
                Songs = new List<Song>() 
        {
            new Song()
            {
                SongName = "Dirty Deeds Done Dirt Cheap",
                SongLength = "4:11"
            },
            new Song()
            {
                SongName = "Love at First Feel",
                SongLength = "3:10"
            }
        }
            };

            // serialize to string            
            string json2 = JsonConvert.SerializeObject(album, Formatting.Indented);

            Console.WriteLine(json2);

            // make sure we can serialize back
            var album2 = JsonConvert.DeserializeObject<Album>(json2);

            Assert.IsNotNull(album2);
            Assert.IsTrue(album2.AlbumName == "Dirty Deeds Done Dirt Cheap");
            Assert.IsTrue(album2.Songs.Count == 2);
        }

        string jsonString = @"[
{
""Id"": ""b3ec4e5c"",
""AlbumName"": ""Dirty Deeds Done Dirt Cheap"",
""Artist"": ""AC/DC"",
""YearReleased"": 1976,
""Entered"": ""2012-03-16T00:13:12.2810521-10:00"",
""AlbumImageUrl"": ""http://ecx.images-amazon.com/images/I/61kTaH-uZBL._AA115_.jpg"",
""AmazonUrl"": ""http://www.amazon.com/gp/product/B00008BXJ4/ref=as_li_ss_tl?ie=UTF8&tag=westwindtechn-20&linkCode=as2&camp=1789&creative=390957&creativeASIN=B00008BXJ4"",
""Songs"": [
    {
    ""AlbumId"": ""b3ec4e5c"",
    ""SongName"": ""Dirty Deeds Done Dirt Cheap"",
    ""SongLength"": ""4:11""
    },
    {
    ""AlbumId"": ""b3ec4e5c"",
    ""SongName"": ""Love at First Feel"",
    ""SongLength"": ""3:10""
    },
    {
    ""AlbumId"": ""b3ec4e5c"",
    ""SongName"": ""Big Balls"",
    ""SongLength"": ""2:38""
    }
]
},
{
""Id"": ""67280fb8"",
""AlbumName"": ""Echoes, Silence, Patience & Grace"",
""Artist"": ""Foo Fighters"",
""YearReleased"": 2007,
""Entered"": ""2012-03-16T00:13:12.2810521-10:00"",
""AlbumImageUrl"": ""http://ecx.images-amazon.com/images/I/41mtlesQPVL._SL500_AA280_.jpg"",
""AmazonUrl"": ""http://www.amazon.com/gp/product/B000UFAURI/ref=as_li_ss_tl?ie=UTF8&tag=westwindtechn-20&linkCode=as2&camp=1789&creative=390957&creativeASIN=B000UFAURI"",
""Songs"": [
    {
    ""AlbumId"": ""67280fb8"",
    ""SongName"": ""The Pretender"",
    ""SongLength"": ""4:29""
    },
    {
    ""AlbumId"": ""67280fb8"",
    ""SongName"": ""Let it Die"",
    ""SongLength"": ""4:05""
    },
    {
    ""AlbumId"": ""67280fb8"",
    ""SongName"": ""Erase/Replay"",
    ""SongLength"": ""4:13""
    }
]
},
{
""Id"": ""7b919432"",
""AlbumName"": ""End of the Silence"",
""Artist"": ""Henry Rollins Band"",
""YearReleased"": 1992,
""Entered"": ""2012-03-16T00:13:12.2800521-10:00"",
""AlbumImageUrl"": ""http://ecx.images-amazon.com/images/I/51FO3rb1tuL._SL160_AA160_.jpg"",
""AmazonUrl"": ""http://www.amazon.com/End-Silence-Rollins-Band/dp/B0000040OX/ref=sr_1_5?ie=UTF8&qid=1302232195&sr=8-5"",
""Songs"": [
    {
    ""AlbumId"": ""7b919432"",
    ""SongName"": ""Low Self Opinion"",
    ""SongLength"": ""5:24""
    },
    {
    ""AlbumId"": ""7b919432"",
    ""SongName"": ""Grip"",
    ""SongLength"": ""4:51""
    }
]
}
]";
    }


    public class Song
    {
        public string SongName { get; set; }
        public string SongLength { get; set; }
    }

    public class Album
    {
        public DateTime Entered { get; set; }
        public string AlbumName { get; set; }
        public string Artist { get; set; }
        public int YearReleased { get; set; }
        public List<Song> Songs { get; set; }
    }
}
